#****************************************************************************
#*                             IverilogCmd.tcl
#*
#* Author: Matthew Ballance
#* Desc:   Implements vvp-specific TCL commands
#* <Copyright> (c) 2001-2003 Matthew Ballance (mballance@users.sourceforge.net)
#*
#*    This source code is free software; you can redistribute it
#*    and/or modify it in source code form under the terms of the GNU
#*    General Public License as published by the Free Software
#*    Foundation; either version 2 of the License, or (at your option)
#*    any later version.
#*
#*    This program is distributed in the hope that it will be useful,
#*    but WITHOUT ANY WARRANTY; without even the implied warranty of
#*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*    GNU General Public License for more details.
#*
#*    You should have received a copy of the GNU General Public License
#*    along with this program; if not, write to the Free Software
#*    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#*
#* </Copyright>
#****************************************************************************


#********************************************************************
#* iverilog
#********************************************************************
proc iverilog args {
    global IVI_HOME
    global tcl_platform

    set targs $args

    set options ""
    set arguments ""
    set out_file  ""

    if {[llength $args] == 0} {
        set ret [IviProc::ivi_exec iverilog]
        return
    }

    while {[llength $targs] > 0} {
        set arg [lindex $targs 0]
        set shift 1

        if {[regexp {^-} $arg]} {
            set shift 2
            lappend options $arg [lindex $targs 1]

            if {$arg == "-o"} {
                set out_file [lindex $targs 1]
            }
        } else {
            lappend arguments $arg
        }

        set targs [lrange $targs $shift end]
    }

    if {$out_file == ""} {
        set out_file "a.out"
    }

    if {[IviConfig::current IVerilog.vlib_dirs] != ""} {
        foreach vlib_dir [IviConfig::current IVerilog.vlib_dirs] {
            ivi_puts "Adding Verilog Library dir \"$vlib_dir\""
            lappend options -y $vlib_dir
        }
    }

    if {[IviConfig::current IVerilog.inc_dirs] != ""} {
        foreach inc_dir [IviConfig::current IVerilog.inc_dirs] {
            ivi_puts "Adding Verilog Include path \"$inc_dir\""
            lappend options -y $inc_dir
        }
    }

    set cmd_arglist "iverilog"

    foreach opt $options {
        lappend cmd_arglist $opt
    }

    foreach arg $arguments {
        lappend cmd_arglist $arg
    }

    set ret [IviProc::ivi_exec $cmd_arglist]
}


